@@ -2,7 +2,9 @@ package HTTP::Proxy::BodyFilter::tags;
 
 use strict;
 use Carp;
-use base qw( HTTP::Proxy::BodyFilter );
+use HTTP::Proxy::BodyFilter;
+use vars qw( @ISA );
+@ISA = qw( HTTP::Proxy::BodyFilter );
 
 =head1 NAME
 
@@ -47,6 +49,10 @@ Philippe "BooK" Bruhat, E<lt>book@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
+Copyright 2003-2004, Philippe Bruhat
+
+=head1 LICENSE
+
 This module is free software; you can redistribute it or modify it under
 the same terms as Perl itself.
 
